//
(function($){
	var recordsets = {},
		links = {},
		url = document.location.href;

	function log(err)
	{
		if('console' in window) console.log(err);
	}

	function populate_recordset(recordset)
	{
		var settings = recordsets[recordset]['settings'],
			data = {'cf7_recordset':recordset, 'cf7' : settings.cf7},
			v, f;

		if('connection' in recordsets[recordset]) recordsets[recordset].connection.abort();

		if('variables' in settings && settings.variables.length)
		{
			data['variables'] = {};
			for(var i in settings.variables)
			{
				v = get_variable(settings.variables[i]);
				if(typeof v != 'undefined') data['variables'][settings.variables[i]] = v;
			}
		}

		if('fields' in settings && settings.fields.length)
		{
			data['fields'] = {};
			for(var i in settings.fields)
			{
				f = get_field(settings.fields[i]);
				if(f) data['fields'][settings.fields[i]] = get_value(f);
			}
		}

		recordsets[recordset]['connection'] = $.ajax(
			{
				dataType : 'json',
				url : url,
				cache : false,
				data : data,
				success : (function( recordset ){
					return function( response ){
						if(!('error' in response))
						{
							recordsets[recordset]['data'] = response.data;
							$(document).trigger('cf7-recordset', [recordset, response.data]);
							if(typeof CustomEvent != 'undefine')
							{
								try
								{
									var evt = new CustomEvent(
										'cf7-recordset',
										{
											detail:
											{
												'recordset-id': recordset,
												'recordset-data': response.data
											}
										}
									);
									document.dispatchEvent(evt);
								}
								catch(err){log(err);}
							}
						}
						else log('Populate Recordset: '+response['error']);
					};
				})(recordset)
			}
		);

	}

	function populate_field(recordset, field_name)
	{
		var field = get_field(field_name);
		if(field)
		{
			var settings = links[recordset][field_name],
				data 	 = recordsets[recordset]['data'],
				type 	 = get_type(field);

			// Get valid data
			var records=[],
				record,
				t 	= ('text' in settings) ? $.trim( settings.text  ) : '',
				v 	= ('value' in settings) ? $.trim( settings.value  ) : '',
				c 	= ('condition' in settings) ? $.trim( settings.condition  ) : '';

			if( c != '' ) c = replace_variables(c);

			for( var i in data )
			{
				record = data[i];
				try{
					if( c == '' || eval(c))
					{
						var item = {};
						if(t != '' && typeof record[t] != 'undefined' ) item['text' ] = record[t];
						if(v != '' && typeof record[v] != 'undefined' ) item['value'] = record[v];
						if(!$.isEmptyObject(item)) records.push( item );
					}
				}catch(err){}
			}

			switch(type)
			{
				case 'simple': populate_simple(field, records); break;
				case 'checkbox': populate_checkbox(field, records); break;
				case 'radio': populate_radio(field, records); break;
				case 'list': populate_list(field, records); break;
			}

			field = get_field(field_name);
			if(field) field.change();
		}
	}

	function replace_variables(str)
	{
		var field_regexp = new RegExp('\{field\.([^\}]+)\}','i'),
			var_regexp   = new RegExp('\{var\.([^\}]+)\}','i'),
			match, field, value;

		while(match = field_regexp.exec(str))
		{
			value = '';
			field = get_field(match[1]);
			if(field) value = get_value(field);
			str = str.replace( match[0], value );
		}

		while(match = var_regexp.exec(str))
		{
			value = get_variable(match[1]);
			value = value || '';
			str = str.replace( match[0], value );
		}

		return str;
	}

	function get_field(field_name)
	{
		var field = $('[name="'+field_name+'"]');
		if(field.length == 0) field = $('[name="'+field_name+'[]"]'); // Checkbox and radio buttons.
		if(field.length == 0) field = $('.'+field_name); // Theh field's container.
		return (field.length) ? field : false;
	}

	function get_type(field)
	{
		var tagName = field[0].tagName.toUpperCase(),
			type = 'simple';
		if(tagName == 'INPUT')
		{
			var t = field.attr('type').toLowerCase();
			if(t == 'checkbox' || t == 'radio')
			{
				type = t;
			}
		}
		else if(tagName == 'SELECT') type = 'list';
		return type;
	}

	function get_value(field)
	{
		var type = get_type(field);
		switch(type)
		{
			case 'simple': return field.val();
			case 'radio': return field.filter(':checked').val();
			case 'checkbox':
				var values = [];
				field.filter(':checked').each(function(){values.push(this.value);});
				return values;
			case 'list':
				var values = [];
				field.find(':selected').each(function(){values.push(this.value);});
				return values;
		}
	}

	function get_variable(variable)
	{
		if(variable in window) return window[variable];
		return undefined;
	}

	function populate_simple(field, data)
	{
		var value = '';
		if(data.length && 'value' in data[0])
		{
			value = data[0].value;
		}
		field.val(value);
	}

	function populate_radio(field, data)
	{
		var checked_value = get_value(field),
			name = field.attr('name'),
			parent = field.closest('.wpcf7-radio'),
			radio = '', value, text, position, checked,
			h = data.length;

		if(h)
		{
			for(var i = 0; i < h; i++)
			{
				position = (i == 0) ? 'first' : ((i == h-1) ? 'last' : '');
				value = (new  String(data[i]['value'])).replace(/\"/g, '\\"');
				text = data[i]['text'];
				checked = (data[i]['value'] == checked_value) ? 'CHECKED' : '';
				radio += '<span class="wpcf7-list-item '+position+'">'+
					'<input type="radio" name="'+name+'" value="'+value+'" '+checked+'>'+
					'<span class="wpcf7-list-item-label">'+text+'</span></span>';
			}
		}
		else radio += '<span class="wpcf7-list-item first"><input type="radio" name="'+name+'" disabled></span>';
		parent.html(radio);
	}

	function populate_checkbox(field, data)
	{
		var checked_values = get_value(field),
			name = field.attr('name'),
			parent = field.closest('.wpcf7-checkbox'),
			checkbox = '', value, text, position, checked,
			h = data.length;

		if(h)
		{
			for(var i = 0; i < h; i++)
			{
				position = (i == 0) ? 'first' : ((i == h-1) ? 'last' : '');
				value = (new  String(data[i]['value'])).replace(/\"/g, '\\"');
				text = data[i]['text'];
				checked = (checked_values.indexOf(data[i]['value']) != -1) ? 'CHECKED' : '';
				checkbox += '<span class="wpcf7-list-item '+position+'">'+
					'<input type="checkbox" name="'+name+'" value="'+value+'" '+checked+'>'+
					'<span class="wpcf7-list-item-label">'+text+'</span></span>';
			}
		}
		else checkbox += '<span class="wpcf7-list-item first"><input type="checkbox" name="'+name+'" disabled></span>';
		parent.html(checkbox);
	}

	function populate_list(field, data)
	{
		var selected_values = get_value(field), options = '', selected, value;
		for(var i in data)
		{
			value = (new  String(data[i]['value'])).replace(/\"/g, '\\"');
			selected = (selected_values.indexOf(data[i]['value']) != -1) ? 'SELECTED' : '';
			options += '<option value="'+value+'" '+selected+'>'+data[i]['text']+'</option>';
		}
		field.html(options);
	}

	function fields_to_recordset(recordset, settings)
	{
		if('fields' in settings)
		{
			for(var i in settings.fields)
			{
				$(document).on(
					'change',
					'[name="'+settings.fields[i]+'"]',
					(
						function(recordset)
						{
							return function(){populate_recordset(recordset);};
						})(recordset)
				);
			}
		}
		populate_recordset(recordset);
	}

	function recordset_to_field(recordset, field)
	{
		$(document).on('cf7-recordset', (function(recordset, field){
			return function(evt, name, data){
				if(recordset == name)
				{
					populate_field(recordset, field);
				}
			};
		})(recordset, field));
	}

	function field_to_field(field, fields_list, recordset)
	{
		for(var i in fields_list)
		{
			$(document).on(
				'change',
				'[name="'+fields_list[i]+'"],[name="'+fields_list[i]+'[]"]',
				(
					function(recordset, field){
						return function(evt){
							populate_field(recordset, field);
						};
					}
				)(recordset, field));
		}
	}

	function unescape_settings(obj)
	{
		var d = $('<div></div>');
		if(typeof obj == 'object') for(var i in obj) obj[i] = unescape_settings(obj[i]);
		else obj = d.html(obj).text();
		return obj;
	}
	window['cf7_datasource_register_recordset'] = function(id, settings){
		settings = unescape_settings(settings);
		recordsets[id] = {settings:settings};
		fields_to_recordset(id, settings);
	};

	window['cf7_datasource_register_link'] = function(settings){
		settings = unescape_settings(settings);
		if(
			'recordset' in settings &&
			'field' in settings
		)
		{
			var rs = settings['recordset'],
				f  = settings['field'];

			if(!(rs in links)) links[rs] = {};
			links[rs][f] = settings;
			recordset_to_field(rs, f);
			field_to_field(f, settings['fields'], rs);
		}
		else log('Register Link: recordset and field are required');
	};
})(jQuery)